static const char select_table_sql[] =
"SELECT\n"
"	  not_null_bit\n"
"\n"
"	, not_null_char\n"
"	, not_null_varchar\n"
"\n"
"	, not_null_datetime\n"
"	, not_null_smalldatetime\n"
"\n"
"	, not_null_money\n"
"	, not_null_smallmoney\n"
"\n"
"	, not_null_float\n"
"	, not_null_real\n"
"\n"
"	, not_null_decimal\n"
"	, not_null_numeric\n"
"\n"
"	, not_null_int\n"
"	, not_null_smallint\n"
"	, not_null_tinyint\n"
"\n"
"	, nullable_char\n"
"	, nullable_varchar\n"
"\n"
"	, nullable_datetime\n"
"	, nullable_smalldatetime\n"
"\n"
"	, nullable_money\n"
"	, nullable_smallmoney\n"
"\n"
"	, nullable_float\n"
"	, nullable_real\n"
"\n"
"	, nullable_decimal\n"
"	, nullable_numeric\n"
"\n"
"	, nullable_int\n"
"	, nullable_smallint\n"
"	, nullable_tinyint\n"
"FROM all_types_bcp_unittest";
